<?php
/*
Plugin Name: Dropshipping with Ebay for WooCommerce (Lite version)
Plugin URI: https://ali2woo.com/dropshipping-plugin-ebay/
Description: Dropshipping with Ebay for WooCommerce (Lite version) is a WordPress plugin created for Ebay Drop Shipping and Affiliate marketing
Text Domain: dropshipping-with-ebay-for-woocommerce
Version: 1.2.7
Author: MA-Group
License: GPLv2+
Tested up to: 5.9
WC tested up to: 6.1
Requires PHP: 7.0
WC requires at least: 5.0
 */

if (!class_exists('E2WL_Main')) {

    class E2WL_Main
    {
        /**
         * @var The single instance of the class
         */
        protected static $_instance = null;

        /**
         * @var string Ebay2Woo Lite plugin version
         */
        public $version;

        /**
         * @var string Ebay2Woo Lite plugin version
         */
        public $plugin_name;

        /**
         * @var string path to Ebay2Woo Lite plugin root url
         */
        public $plugin_url;

        /**
         * @var string path to Ebay2Woo Lite plugin root dir
         */
        public $plugin_path;

        /**
         * @var string chrome extension url
         */
        public $chrome_url = 'https://chrome.google.com/webstore/detail/faieahckjkcpljkaedbjidlhhcigddal';

        public static function instance()
        {
            if (is_null(self::$_instance)) {
                self::$_instance = new self();
            }
            return self::$_instance;
        }

        private function __construct()
        {
            require_once ABSPATH . 'wp-admin/includes/plugin.php';
            $plugin_data = get_plugin_data(__FILE__);

            $this->version = $plugin_data['Version'];
            $this->plugin_name = plugin_basename(__FILE__);
            $this->plugin_url = plugin_dir_url(__FILE__);
            $this->plugin_path = plugin_dir_path(__FILE__);

            include_once dirname(__FILE__) . '/includes/settings.php';
            include_once dirname(__FILE__) . '/includes/functions.php';

            include_once dirname(__FILE__) . '/includes/init.php';
            E2WL_Init::init_classes($this->plugin_path . 'includes/classes', 'e2wl_init');
            E2WL_Init::init_addons($this->plugin_path . 'addons');

            if (!class_exists('Requests')) {
                include_once dirname(__FILE__) . '/includes/libs/Requests/Requests.php';
                Requests::register_autoloader();
            }

            register_activation_hook(__FILE__, array($this, 'install'));
            register_deactivation_hook(__FILE__, array($this, 'uninstall'));

            add_action('admin_menu', array($this, 'admin_menu'));

            add_action('admin_enqueue_scripts', array($this, 'admin_assets'));

            add_action('wp_enqueue_scripts', array($this, 'assets'));
        }

        public function install()
        {
            do_action('e2wl_install');
        }

        public function uninstall()
        {
            do_action('e2wl_uninstall');
        }

        public function assets($page)
        {
            do_action('e2wl_assets', $page);
        }

        public function admin_assets($page)
        {
            do_action('e2wl_admin_assets', $page);
        }

        public function admin_menu()
        {
            do_action('e2wl_before_admin_menu');

            add_menu_page(__('Ebay2Woo Lite', 'dropshipping-with-ebay-for-woocommerce'), __('Ebay2Woo Lite', 'dropshipping-with-ebay-for-woocommerce'), 'manage_options', 'e2wl_dashboard', '', plugins_url('assets/img/icon.png', __FILE__));

            do_action('e2wl_init_admin_menu', 'e2wl_dashboard');
        }

    }

}

/**
 * Returns the main instance of E2WL_Main to prevent the need to use globals.
 *
 * @return E2WL_Main
 */
if (!function_exists('E2WL')) {
    function E2WL()
    {
        return E2WL_Main::instance();
    }
}

$ebay2woo_lite = E2WL();

/**
 * Ebay2Woo Lite global init action
 */
do_action('e2wl_init');
